#ifndef _PyGSL_NUMPY_API_VERSION_H_
#define _PyGSL_NUMPY_API_VERSION_H_ 1
/*
 * separate file: used also by ufunc module. does not import
 * standard helpers
 */
#define NPY_NO_DEPRECATED_API NPY_2_0_API_VERSION

#endif  /* _PyGSL_NUMPY_API_VERSION_H_ */
